<?php

namespace Bixo\Note\Policies;

use Litepie\User\Contracts\UserPolicy;
use Bixo\Note\Models\Note;

class NotePolicy
{

    /**
     * Determine if the given user can view the note.
     *
     * @param UserPolicy $user
     * @param Note $note
     *
     * @return bool
     */
    public function view(UserPolicy $user, Note $note)
    {
        if ($user->canDo('note.note.view') && $user->isAdmin()) {
            return true;
        }

        return $note->user_id == user_id() && $note->user_type == user_type();
    }

    /**
     * Determine if the given user can create a note.
     *
     * @param UserPolicy $user
     * @param Note $note
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('note.note.create');
    }

    /**
     * Determine if the given user can update the given note.
     *
     * @param UserPolicy $user
     * @param Note $note
     *
     * @return bool
     */
    public function update(UserPolicy $user, Note $note)
    {
        if ($user->canDo('note.note.edit') && $user->isAdmin()) {
            return true;
        }

        return $note->user_id == user_id() && $note->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given note.
     *
     * @param UserPolicy $user
     * @param Note $note
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Note $note)
    {
        return $note->user_id == user_id() && $note->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given note.
     *
     * @param UserPolicy $user
     * @param Note $note
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Note $note)
    {
        if ($user->canDo('note.note.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given note.
     *
     * @param UserPolicy $user
     * @param Note $note
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Note $note)
    {
        if ($user->canDo('note.note.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
